# coding: utf-8
require 'dread'

module Dread
  class EndpointEvent < Event
    attr_reader    :epnum
    attr_reader    :type
    attr_reader    :status
    attr_reader    :parameters

    DEPEVT_XFERCOMPLETE = 0x01
    DEPEVT_XFERINPROGRESS = 0x02
    DEPEVT_XFERNOTREADY = 0x03
    DEPEVT_RXTXFIFOEVT = 0x04
    DEPEVT_STREAMEVT = 0x06
    DEPEVT_EPCMDCMPLT = 0x07

    STREAMEVT_FOUND = 1
    STREAMEVT_NOTFOUND = 2

    CONTROL_DATA = (1 << 0)
    CONTROL_STATUS = (2 << 0)
    
    TRANSFER_ACTIVE = (1 << 3)

    XFERCOMPLETE_LST = (1 << 3)
    XFERCOMPLETE_IOC = (1 << 2)
    XFERCOMPLETE_SHORT = (1 << 1)

    def initialize(line)
      super

      @event_type = {
        DEPEVT_XFERCOMPLETE => "Transfer Complete",
        DEPEVT_XFERINPROGRESS => "Transfer In Progress",
        DEPEVT_XFERNOTREADY => "Transfer Not Ready",
        DEPEVT_RXTXFIFOEVT => "FIFO Event",
        DEPEVT_STREAMEVT => "Stream",
        DEPEVT_EPCMDCMPLT => "Command Complete",
      }

      decode
    end

    def decode
      @event_data = super

      @epnum = field(@event_data, 5, 1)
      @type = field(@event_data, 9, 6)
      @status = field(@event_data, 15, 12)
      @parameters = field(@event_data, 31, 16)
    end

    def to_s
      str = super

      str += ": #{endpoint}: "
      str += @event_type[@type]

      case @type
      when DEPEVT_XFERCOMPLETE
        str += xfer_complete_status
      when DEPEVT_XFERNOTREADY
        str += xfer_not_ready_status
      when DEPEVT_STREAMEVT
        str += stream_extras
      end

      str
    end

    def endpoint
      num = @epnum >> 1
      dir = (@epnum & 1) == 1 ? "in" : "out"

      "ep#{num}#{dir}"
    end

    private

    def xfer_complete_status
      str = " ["

      if ((@status & XFERCOMPLETE_SHORT) == XFERCOMPLETE_SHORT)
        str += "S"
      else
        str += "s"
      end

      if ((@status & XFERCOMPLETE_IOC) == XFERCOMPLETE_IOC)
        str += "I"
      else
        str += 'i'
      end

      if ((@status & XFERCOMPLETE_LST) == XFERCOMPLETE_LST)
        str += "L"
      else
        str += "l"
      end

      str += "] "
      str
    end

    def xfer_not_ready_status
      str = ""

      if ((@status & TRANSFER_ACTIVE) == TRANSFER_ACTIVE)
        str += " (Active)"
      else
        str += " (Not Active)"
      end

      if @epnum == 0 || @epnum == 1
        case (@status & (3 << 0))
        when CONTROL_DATA
          str += " [ Data Phase ]"
        when CONTROL_STATUS
          str += " [ Status Phase ]"
        end
      end

      str
    end

    def stream_extras
      case @status
      when STREAMEVT_FOUND
        "#{@parameters} Found"
      when STREAMEVT_NOTFOUND
        "#{@parameters} Not Found"
      end
    end
  end
end
